@@ -3,7 +3,7 @@ import socket
 import struct
 from collections import defaultdict, deque
 from hashlib import md5
-from io import TextIOBase
+from io import IOBase, TextIOBase
 from itertools import count
 from struct import Struct
 
@@ -16,7 +16,14 @@ from pg8000.converters import (
     make_params,
     string_in,
 )
-from pg8000.exceptions import DatabaseError, InterfaceError
+from pg8000.exceptions import DatabaseError, Error, InterfaceError
+
+try:
+    from importlib.metadata import version
+except ImportError:
+    from importlib_metadata import version
+
+ver = version("pg8000")
 
 
 def pack_funcs(fmt):
@@ -271,7 +278,7 @@ class CoreConnection:
             try:
                 self._sock.flush()
             except OSError as e:
-                raise InterfaceError("network error on flush") from e
+                raise InterfaceError("network error") from e
 
         self._flush = sock_flush
 
@@ -279,7 +286,7 @@ class CoreConnection:
             try:
                 return self._sock.read(b)
             except OSError as e:
-                raise InterfaceError("network error on read") from e
+                raise InterfaceError("network error") from e
 
         self._read = sock_read
 
@@ -287,7 +294,7 @@ class CoreConnection:
             try:
                 self._sock.write(d)
             except OSError as e:
-                raise InterfaceError("network error on write") from e
+                raise InterfaceError("network error") from e
 
         self._write = sock_write
         self._backend_key_data = None
@@ -334,12 +341,24 @@ class CoreConnection:
         self._write(val)
         self._flush()
 
-        code = self.error = None
-        while code not in (READY_FOR_QUERY, ERROR_RESPONSE):
-            code, data_len = ci_unpack(self._read(5))
-            self.message_types[code](self._read(data_len - 4), None)
-        if self.error is not None:
-            raise self.error
+        try:
+
+            code = self.error = None
+            while code not in (READY_FOR_QUERY, ERROR_RESPONSE):
+
+                try:
+                    code, data_len = ci_unpack(self._read(5))
+                except struct.error as e:
+                    raise InterfaceError("network error") from e
+
+                self.message_types[code](self._read(data_len - 4), None)
+
+            if self.error is not None:
+                raise self.error
+
+        except Error as e:
+            self.close()
+            raise e
 
         self.in_transaction = False
 
@@ -421,35 +440,55 @@ class CoreConnection:
 
         if context.stream is None:
             raise InterfaceError(
-                "An input stream is required for the COPY IN response."
+                "The 'stream' parameter is required for the COPY IN response. The "
+                "'stream' parameter can be an I/O stream or an iterable."
             )
 
-        elif isinstance(context.stream, TextIOBase):
-            if is_binary:
-                raise InterfaceError(
-                    "The COPY IN stream is binary, but the stream parameter is text."
-                )
+        if isinstance(context.stream, IOBase):
+            if isinstance(context.stream, TextIOBase):
+                if is_binary:
+                    raise InterfaceError(
+                        "The COPY IN stream is binary, but the stream parameter is a "
+                        "text stream."
+                    )
 
+                else:
+
+                    def ri(bffr):
+                        bffr.clear()
+                        bffr.extend(
+                            context.stream.read(4096).encode(self._client_encoding)
+                        )
+                        return len(bffr)
+
+                    readinto = ri
             else:
+                readinto = context.stream.readinto
 
-                def ri(bffr):
-                    bffr.clear()
-                    bffr.extend(context.stream.read(4096).encode(self._client_encoding))
-                    return len(bffr)
+            bffr = bytearray(8192)
+            while True:
+                bytes_read = readinto(bffr)
+                if bytes_read == 0:
+                    break
+                self._write(COPY_DATA)
+                self._write(i_pack(bytes_read + 4))
+                self._write(bffr[:bytes_read])
+                self._flush()
 
-                readinto = ri
         else:
-            readinto = context.stream.readinto
+            for k in context.stream:
+                if isinstance(k, str):
+                    if is_binary:
+                        raise InterfaceError(
+                            "The COPY IN stream is binary, but the stream parameter "
+                            "is an iterable with str type items."
+                        )
+                    b = k.encode(self._client_encoding)
+                else:
+                    b = k
 
-        bffr = bytearray(8192)
-        while True:
-            bytes_read = readinto(bffr)
-            if bytes_read == 0:
-                break
-            self._write(COPY_DATA)
-            self._write(i_pack(bytes_read + 4))
-            self._write(bffr[:bytes_read])
-            self._flush()
+                self._send_message(COPY_DATA, b)
+                self._flush()
 
         # Send CopyDone
         self._write(COPY_DONE_MSG)
@@ -472,19 +511,21 @@ class CoreConnection:
         This function is part of the `DBAPI 2.0 specification
         <http://www.python.org/dev/peps/pep-0249/>`_.
         """
+
+        if self._usock is None:
+            raise InterfaceError("connection is closed")
+
         try:
             self._write(TERMINATE_MSG)
             self._flush()
-            self._sock.close()
-        except AttributeError:
-            raise InterfaceError("connection is closed")
-        except ValueError:
-            raise InterfaceError("connection is closed")
-        except socket.error:
-            pass
         finally:
-            self._usock.close()
-            self._sock = None
+            try:
+                self._usock.close()
+            except socket.error as e:
+                raise InterfaceError("network error") from e
+            finally:
+                self._sock = None
+                self._usock = None
 
     def handle_AUTHENTICATION_REQUEST(self, data, context):
         """https://www.postgresql.org/docs/current/protocol-message-formats.html"""
@@ -756,7 +797,7 @@ class CoreConnection:
             try:
                 code, data_len = ci_unpack(self._read(5))
             except struct.error as e:
-                raise InterfaceError("network error on read") from e
+                raise InterfaceError("network error") from e
 
             self.message_types[code](self._read(data_len - 4), context)
 
