@@ -3,7 +3,6 @@ from itertools import count, islice
 from time import localtime
 from warnings import warn
 
-import pg8000
 from pg8000.converters import (
     BIGINT,
     BOOLEAN,
@@ -41,14 +40,11 @@ from pg8000.converters import (
     VARCHAR_ARRAY,
     XID,
 )
-from pg8000.core import Context, CoreConnection
+from pg8000.core import Context, CoreConnection, ver
 from pg8000.exceptions import DatabaseError, Error, InterfaceError
 
 
-from ._version import get_versions
-
-__version__ = get_versions()["version"]
-del get_versions
+__version__ = ver
 
 # Copyright (c) 2007-2009, Mathieu Fenniak
 # Copyright (c) The Contributors
@@ -446,13 +442,16 @@ class Cursor:
         """
         try:
             if not self._c.in_transaction and not self._c.autocommit:
-                self._c.execute_unnamed("begin transaction")
+                self._c.execute_simple("begin transaction")
 
-            statement, vals = convert_paramstyle(paramstyle, operation, args)
+            if len(args) == 0 and stream is None:
+                self._context = self._c.execute_simple(operation)
+            else:
+                statement, vals = convert_paramstyle(paramstyle, operation, args)
+                self._context = self._c.execute_unnamed(
+                    statement, vals=vals, oids=self._input_oids, stream=stream
+                )
 
-            self._context = self._c.execute_unnamed(
-                statement, vals=vals, oids=self._input_oids, stream=stream
-            )
             if self._context.rows is None:
                 self._row_iter = None
             else:
@@ -615,7 +614,7 @@ class Cursor:
                 try:
                     oid, _ = self._c.py_types[size]
                 except KeyError:
-                    oid = pg8000.converters.UNKNOWN
+                    oid = UNKNOWN
             oids.append(oid)
 
         self._input_oids = oids
