@@ -45,7 +45,7 @@ from pg8000.converters import (
     pg_interval_in as pginterval_in,
     pg_interval_out as pginterval_out,
 )
-from pg8000.core import Context, CoreConnection
+from pg8000.core import Context, CoreConnection, ver
 from pg8000.dbapi import (
     BINARY,
     Binary,
@@ -64,11 +64,7 @@ from pg8000.dbapi import (
 )
 from pg8000.exceptions import DatabaseError, Error, InterfaceError
 
-
-from ._version import get_versions
-
-__version__ = get_versions()["version"]
-del get_versions
+__version__ = ver
 
 # Copyright (c) 2007-2009, Mathieu Fenniak
 # Copyright (c) The Contributors
@@ -243,13 +239,15 @@ class Cursor:
         """
         try:
             if not self._c.in_transaction and not self._c.autocommit:
-                self._c.execute_unnamed("begin transaction")
+                self._c.execute_simple("begin transaction")
 
-            statement, vals = convert_paramstyle(self.paramstyle, operation, args)
-
-            self._context = self._c.execute_unnamed(
-                statement, vals=vals, oids=self._input_oids, stream=stream
-            )
+            if len(args) == 0 and stream is None:
+                self._context = self._c.execute_simple(operation)
+            else:
+                statement, vals = convert_paramstyle(self.paramstyle, operation, args)
+                self._context = self._c.execute_unnamed(
+                    statement, vals=vals, oids=self._input_oids, stream=stream
+                )
 
             rows = [] if self._context.rows is None else self._context.rows
             self._row_iter = iter(rows)
