@@ -36,11 +36,14 @@ from pg8000.converters import (
     VARCHAR,
     VARCHAR_ARRAY,
     XID,
+    identifier,
+    literal,
     make_params,
 )
-from pg8000.core import CoreConnection
+from pg8000.core import CoreConnection, ver
 from pg8000.exceptions import DatabaseError, Error, InterfaceError
 
+__version__ = ver
 
 # Copyright (c) 2007-2009, Mathieu Fenniak
 # Copyright (c) The Contributors
@@ -192,12 +195,14 @@ class Connection(CoreConnection):
         return context.row_count
 
     def run(self, sql, stream=None, types=None, **params):
-        statement, make_vals = to_statement(sql)
-        oids = () if types is None else make_vals(defaultdict(lambda: None, types))
-
-        self._context = self.execute_unnamed(
-            statement, make_vals(params), oids=oids, stream=stream
-        )
+        if len(params) == 0 and stream is None:
+            self._context = self.execute_simple(sql)
+        else:
+            statement, make_vals = to_statement(sql)
+            oids = () if types is None else make_vals(defaultdict(lambda: None, types))
+            self._context = self.execute_unnamed(
+                statement, make_vals(params), oids=oids, stream=stream
+            )
         return self._context.rows
 
     def prepare(self, sql):
@@ -269,4 +274,6 @@ __all__ = [
     "VARCHAR",
     "VARCHAR_ARRAY",
     "XID",
+    "identifier",
+    "literal",
 ]
